import os, sys, random
from posixpath import dirname
import math
import numpy
from random import randint
from PIL import Image, ImageDraw
import shutil

# tweakable constants for transformations
# Variations in position of the symbol,
# variations in the orientation of the symbol,
# variations in the size of the symbol,
# variations in the thickness of the strokes in the symbol,
# variations in the size and number of stray marks such as ellipsoids drawn in the image.


#transformation_parameters 

position=1
size = 5
thickness = 10
angle_min=-20
angle_max=20
stray_number=2
stray_size=3


#constant values
canvas_size=25
cards=['C','H','D','S']
#to remove if any directory exits with the same name
dir_name =os.getcwd()
k = str(dir_name)+'/'+str(sys.argv[1])

if os.path.isdir(k):
    shutil.rmtree(k)
#Read the command line arguments
l=[]
l=sys.argv

#Check if any arguments are missing
if len(l)!=3:
   print("Arguments missing!")
   exit()

folder= l[1]       # folder where the images will be saved
n_examples= l[2]   # number of cards to be generated

os.makedirs(folder,exist_ok=True)  #create the data folder



def stray_marks(im, canvas_size):
    
    draw = ImageDraw.Draw(im)

    #generate four coordinates
    x1 = randint(0,canvas_size)
    y1 = randint(0,canvas_size)

    x2 = x1 + stray_size*10
    y2 = y1 + stray_size*10

    shape=random.choice(['ellipse','line'])

    if shape=='ellipse':
        #Add a stray ellipse
        draw.ellipse(((x1,y1),(x2,y2)),fill="black")
    else:
        draw.line(((x1,y1),(x2,y2)),fill="black",width=3)

    return im

#Function to draw the clubs symbol
def draw_clubs(n,x,y,size):


        #x and y are added to every coordinate value to give the new position
    
        clubs= [(158.5+x, 120.5+y), (175.5+x, 120.5+y), (196.5+x, 132.5+y),
        (202.5+x, 149.5+y), (201.5+x, 173.5+y), (191.5+x, 186.5+y),
        (211.5+x, 178.5+y), (230.5+x, 170.5+y), (252.5+x, 183.5+y),
        (253.5+x, 205.5+y), (237.5+x, 228.5+y), (219.5+x, 241.5+y),
        (195.5+x, 243.5+y), (181.5+x, 228.5+y), (185.5+x, 271.5+y),
        (169.5+x, 270.5+y), (171.5+x, 233.5+y), (158.5+x, 240.5+y),
        (140.5+x, 248.5+y), (106.5+x, 232.5+y), (98.5+x, 207.5+y),
        (108.5+x, 182.5+y), (136.5+x, 176.5+y), (131.5+x, 165.5+y),
        (120.5+x, 137.5+y), (137.5+x, 122.5+y),(158.5+x, 120.5+y)]

        #changing the size of the symbol by multiplying the size with every coordinate value
        shape = [(k*size,v*size) for k,v in clubs]   
       
        #Finding the maximum coordinate value to adjust the canvas size
        maxval=int(max(max(shape)))    
        path=f"{n}_C.png"

        return shape,maxval,path

    
#Function to draw the heart symbol
def draw_hearts(n,x,y,size):
    
        hearts=[(176.5+x, 155.5+y),(190.5+x, 136.5+y),(212.5+x, 119.5+y),
        (243.5+x, 116.5+y),(267.5+x, 135.5+y),(261.5+x, 170.5+y),
        (247.5+x, 198.5+y),(218.5+x, 228.5+y),(201.5+x, 250.5+y),
        (190.5+x, 269.5+y),(172.5+x, 256.5+y),(146.5+x, 234.5+y),
        (122.5+x, 202.5+y),(107.5+x, 180.5+y),(99.5+x, 150.5+y),
        (107.5+x, 122.5+y),(126.5+x, 108.5+y),(147.5+x, 111.5+y),
        (165.5+x, 134.5+y),(176.5+x, 155.5+y)]

        #changing the size of the symbol
        shape = [(k*size,v*size) for k,v in hearts]   
       
        maxval=int(max(max(shape)))
        path = f"{n}_H.png"
        return shape,maxval,path
    

#Function to draw the diamond symbol
def draw_diamond(n,x,y,size):
    
        diamond=[(174.5+x, 281.5+y),(105.5+x, 197.5+y),
        (178.5+x, 125.5+y),(243.5+x, 195.5+y),(174.5+x, 281.5+y)]
        
        #changing the size of the symbol
        shape = [(k*size,v*size) for k,v in diamond]   
       
        maxval=int(max(max(shape)))
        path = f"{n}_D.png"
        return shape,maxval,path
    

#Function to draw the spades symbol    
def draw_spades(n,x,y,size):
    
        spades=[(175.5+x, 134.5+y), (159.5+x, 157.5+y), (141.5+x, 186.5+y),
        (129.5+x, 226.5+y), (137.5+x, 249.5+y), (165.5+x, 256.5+y),
        (185.5+x, 250.5+y), (190.5+x, 283.5+y), (206.5+x, 283.5+y),
        (203.5+x, 253.5+y), (217.5+x, 258.5+y), (240.5+x, 259.5+y),
        (256.5+x, 244.5+y), (256.5+x, 215.5+y), (235.5+x, 180.5+y),
        (206.5+x, 151.5+y), (186.5+x, 123.5+y),(175.5+x, 134.5+y)]

        #changing the size of the symbol
        shape = [(k*size,v*size) for k,v in spades]   
       
        maxval=int(max(max(shape)))
        path = f"{n}_S.png"
        return shape,maxval,path
    

def draw_card(n,card):

    #generate variables x and y, that determine the position of the symbol
    x=random.randint(-50,50) *position
    y=random.randint(-50,50) *position

    #random angle generated between the given min and max angle
    angle=random.randint(angle_min,angle_max)

    #size of the symbol
    to_size=random.randint(1,size)

    #Thickness of the strokes
    to_thickness=random.randint(5,thickness)

    #Randomly choose to have stray marks or not
    stray=random.choice([True,False])

           
    if card=='C':
        shape,maxval,path=draw_clubs(n,x,y,to_size)

    elif card=='H':
        
        shape,maxval,path=draw_hearts(n,x,y,to_size)

    elif card=='D':
        
        shape,maxval,path=draw_diamond(n,x,y,to_size)

    elif card=='S':
         
         shape,maxval,path=draw_spades(n,x,y,to_size)

    #create an image with the updated canvas size
    im = Image.new("RGB", (maxval+500, maxval+500), "white")
    
    draw = ImageDraw.Draw(im)

    #drawing the symbol on the image
    draw.line(shape, fill="black", width=to_thickness)

    #If stray marks are chosen to be drawn
    
    if stray==True:

        #generate random number of stray marks
        stray_n= random.randint(1,stray_number)
        
        for i in range(stray_n):   
            im=stray_marks(im,maxval+500)

    total_path = os.path.join(folder, path)

    #Change the image orientation to the given angle
    im=im.rotate(angle, fillcolor='white')
    
    im=im.resize((canvas_size,canvas_size))
    im=im.convert("L")
    
    im.save(total_path, "png")
        
    

def start():
    
    for i in range(int(n_examples)):

        #Random generation of cards
        card= random.choice(cards)
        draw_card(i+1,card)

start()



